clc
clear all
close all

%%
%Input info for the simply suported beam
Emodulus = 2.1e+11; %[N/m^2]
Poissons = 0.3;
Density = 7850; %[kg/m^3]
Length = 100; %[m]
width=0.1; %[m]
height=0.1; %[m]
m = Density*width*height; %[kg/m]
I = (1/12)*width*height^3; %[m^4]

% Calculation of relevant factors for the Rayleigh damping
om_p = ((pi/Length)^2)*sqrt(Emodulus*I/m);
om_r = ((10*pi/Length)^2)*sqrt(Emodulus*I/m);
xi1 = 0.05;
xi2 = 0.05;
alpha = (2*om_p*om_r*((om_p*xi2)-(om_r*xi1))/((om_p^2)-(om_r^2)));
beta = 2*((om_p*xi1)-(om_r*xi2))/((om_p^2)-(om_r^2));

% Meshing of the beam
NumberOfElements=10;
NumberOfNodes = NumberOfElements+1;
ElementLength = Length/NumberOfElements;

%%
% The creation of nodes for the beam 
fid=fopen('nodeSet.csv','wt'); 

format='%.0f,%.2f,%.0f \n';
for i=1:2:NumberOfNodes
    node(i)= i;
    x(i) = (i-1)*Length/(NumberOfElements); 
    node(i+1)= i+1;
    x(i+1) =(i*Length/(NumberOfElements)); 
end
for i=1:1:NumberOfNodes
    Nodes(i,:)=[node(i),x(i),0];
    fprintf(fid,format,Nodes(i,:));
end

%%
% Mesh the beam with B23 elements
fid=fopen('elementSet.csv','wt');
for a=1:NumberOfNodes
    mesh(a)= a;
end
format='%.0f,%.0f,%.0f \n';
for j=1:1:NumberOfElements
      element(j,:)=[j,mesh(j),mesh(j+1)]; 
      fprintf(fid,format,element(j,:));
end


%%
%Create input file for the Abaqus model
fid=fopen('SSBMovCar.inp','wt');  
%%
fprintf(fid,'*Preprint, echo=NO, model=YES, history=NO, contact=NO \n');

%% Parts
fprintf(fid,'** \n');
fprintf(fid,'** Model definition');
fprintf(fid,'**\n');

%% Bring in info about nodes
% Nodes
fprintf(fid, '*NODE,NSET=NODES \n');
nodeSet=csvread('nodeSet.csv');

format='     %.0f,     %.5f,     %.0f \n';
for i=1:length(nodeSet)
    node(i)=nodeSet(i,1);
    x(i)=nodeSet(i,2);
    Nodes(i,:)=[node(i),x(i),0];
    fprintf(fid,format,Nodes(i,:));
end

%% Bring in info about elements
% Elements
fprintf(fid, '*Element, type=B23, Elset=wire \n');
elementSet=csvread('elementSet.csv');
format='%.0f,  %.0f,  %.0f \n';
for i=1:length(elementSet)
    element(i)=elementSet(i,1);
    el1(i)=elementSet(i,2);
    el2(i)=elementSet(i,3);
    
    Elements(i,:)=[element(i),el1(i),el2(i)];
    fprintf(fid,format,Elements(i,:));
end


%% Creating Sections
fprintf(fid, '** \n');
fprintf(fid, '*Beam Section, elset=wire, section=rect, material=Material-1 \n')
fprintf(fid, '%.2f, %.2f \n',width, height);

%% Creating Materials
format='%.0f,%.0f \n';
fprintf(fid, '** \n');
fprintf(fid, '*Material, name=Material-1 \n');
fprintf(fid, '*Density \n');
fprintf(fid,'%.2f \n', Density);
fprintf(fid, '*Elastic \n');
fprintf(fid,format, Emodulus,Poissons);
fprintf(fid, '** \n');
fprintf(fid,'%s%.5f, %s%.5f \n', '*Damping, Alpha=',alpha,' Beta=',beta) 
fprintf(fid, '** \n');
%% History data
fprintf(fid, '** History data \n');
fprintf(fid, '** \n');


%% Bring in info about cars
% Cars
carSet=csvread('carSet.csv',1,0);

% The script takes in information for a .csv file with 5 columns containg information on:
% Which car, which axle, distance from axle to first node at time t=0, the force extended from the axle, the speed of the axle

NumOfWheels = size(carSet,1);
for i=1:NumOfWheels
    wheel(i)=i;
    x0(i)=carSet(i,3);
    Force(i)=-carSet(i,4);
    Speed(i)=carSet(i,5);
    Steptime(i)=(Length)/(Speed(i)/3.6);
    t0(i) = x0(i)*Steptime(i)/Length;
    Cars(i,:)=[wheel(i),x0(i),Speed(i),t0(i),Force(i)];
end

t0max = max(t0);
t0min = min(t0);
t0sum = sum(t0);
MeanSteptime = mean(Steptime);


%% Creating Steps
format='%.10f,%.2f,%.10f \n';
fprintf(fid, '** \n');
fprintf(fid,'%s \n', '*STEP, Amplitude=Step, Inc=30000, Name=Car, Nlgeom=no');
fprintf(fid, '*Dynamic \n');
fprintf(fid,'%.5f, %.2f, %s, %.5f \n',.0001*(MeanSteptime+t0min), max(Steptime+t0)+t0min, '1e-12', .001*(MeanSteptime+t0min));  
fprintf(fid, '** \n');

%% Boundary Conditions
fprintf(fid, '*Boundary \n'); 
fprintf(fid,'1, Pinned \n'); % Boundary condition at left support
fprintf(fid,'%.0f, %s \n',NumberOfNodes+NumberOfElements,' 2'); % Boundary condition at right support
fprintf(fid, '** \n');

%% Implementing all the effects inflicted by each axle on the SSB
for c = 1:NumOfWheels
%% Assigning one amplitude for each time step
format='%.3f, %.3f, %.3f, %.3f, %.3f, %.3f \n';
for l=1:1:length(nodeSet)
    amplitude(l)=l;
    amp(l,:)=[amplitude(l)];
    increment(l)=t0(c) + (Steptime(c)*(amp(l)-1)/(length(nodeSet)-1));
    inc(l,:)=[increment(l)];
end

%The first amplitude of the analysis
fprintf(fid,'%s, %s%.0f%s%.0f, %s \n', '*Amplitude', 'Name=Amp-',c,'-',0,'TIME=TOTAL TIME');
fprintf(fid,'%.3f, %.3f, %.3f, %.3f, %.3f, %.3f \n',inc(1),1,inc(2),0,inc(3),0);
    
for m=2:1:length(nodeSet)-3
 
fprintf(fid,'%s, %s%.0f%s%.0f, %s \n', '*Amplitude', 'name=Amp-',c,'-',m,'TIME=TOTAL TIME');
fprintf(fid,format,inc(m),0,inc(m+1),1,inc(m+2),0);

end
% The final amplitude of the analysis
fprintf(fid,'%s, %s%.0f%s%.0f, %s \n', '*Amplitude', 'name=Amp-',c,'-',length(nodeSet)-1,'TIME=TOTAL TIME');
fprintf(fid,'%.3f, %.3f, %.3f, %.3f, %.3f, %.3f \n',inc(length(nodeSet)-2),0,inc(length(nodeSet)-1),1,inc(length(nodeSet)),0);


  
%% Defining Loads and Amplitudes
fprintf(fid,'** \n');

% The starting load in the first node
fprintf(fid,'%s, %s%.0f%s%.0f \n', '*Cload', 'Amp=Amp-',c,'-',0);
fprintf(fid,'%.0f, %s, %.2f \n',1, '2', Force(c));
fprintf(fid,'** \n');

for n=1:1:length(amp)-1    
%Each node is assigned one load with an amplitude that is equal to one at
%time t_n, and zero at all other times
fprintf(fid,'%s, %s%.0f%s%.0f \n', '*Cload', 'Amp=Amp-',c,'-',n-1);
fprintf(fid,'%.0f, %s, %.2f \n',n, '2', Force(c));

% Applying moments in the margin noads of the load
fprintf(fid,'%s, %s%.0f%s%.0f \n', '*Cload', 'Amp=Amp-',c,'-',n-1);
fprintf(fid,'%.0f, %s, %.2f \n',n-1, '6', -ElementLength*Force(c));

fprintf(fid,'%s, %s%.0f%s%.0f \n', '*Cload', 'Amp=Amp-',c,'-',n-1);
fprintf(fid,'%.0f, %s, %.2f \n',n+1, '6', ElementLength*Force(c));
end

% The final load in the last node
fprintf(fid,'%s, %s%.0f%s%.0f \n', '*Cload', 'Amp=Amp-',c,'-',length(amp)-2);
fprintf(fid,'%.0f, %s, %.2f \n',length(amp)-1, '2', Force(c));

end

%% Output
fprintf(fid, '** \n');
fprintf(fid, '** OUTPUT REQUESTS \n');
fprintf(fid, '** \n');
fprintf(fid, '*Restart, write, frequency=0 \n');
fprintf(fid, '** \n');
fprintf(fid, '* El file \n');
fprintf(fid, 'S \n');
fprintf(fid, '* Node file \n');
fprintf(fid, 'U \n');
fprintf(fid, 'A \n');
fprintf(fid, '*End Step \n');

 %% Run analysis in Abaqus
 clc
 clc
 JobName='SSBCarJob';
 system(['abaqus job=' JobName ' input=SSBMovCar interactive']);
  
 pause(2);
  
  winopen([ JobName '.odb']);
  
  %% Is the code working?
  if ans == 0
      fprintf '\n \n \n \n Success! \n'
  else
      fprintf '\n \n \n \n Failure! \n'
  end

